#include "LayerManager.h"
#include "InfoDisplay.h"

#include "NoiseSlice.h"

LayerManager* LayerManager::m_Manager = NULL;

void LayerManager::Initialize(GLUI* parentGlui, int width, int height)
{
	m_parentGlui = parentGlui;
	m_panelLayers = parentGlui->add_panel("Layer controls");
	m_lbLayers = parentGlui->add_listbox_to_panel(m_panelLayers, "Layers: ");

	//add/remove buttons
	parentGlui->add_button_to_panel(m_panelLayers, "Remove layer", GUI_BUTTON_REMOVE, LayerManager::event_RemoveLayer);
	parentGlui->add_button_to_panel(m_panelLayers, "Add new layer", GUI_BUTTON_ADD, LayerManager::event_AddLayer);

	m_width = width;
	m_height = height;

	m_TotalLayerCount = 0;

	ResetCoords();

	m_infoDisplay.Initialize(this, width, height);
}

void LayerManager::event_RemoveLayer(int value)
{
	m_Manager->RemoveLayer();

	//add a redraw event?
}

void LayerManager::event_AddLayer(int value)
{
	m_Manager->AddLayer(new NoiseSlice());

	//add a redraw event?
}

LayerManager::LayerManager(void)
{
	m_Manager = this;

	m_width = 0;
	m_height = 0;

	m_renderableCount = 0;
	//m_currentLayer = -1;
}

LayerManager::~LayerManager(void)
{
	//dispose of layers
	for(int i = 0; i < m_renderableCount; i++)
		delete m_renderableList[i];
}

void LayerManager::MoveCoordinates(int delta_x, int delta_y)
{
	//xdiff, ydiff are in PIXELS
	//we want to translate from pixels to 3-space.

	//REIMPLEMENT
	float xdiff = delta_x, ydiff = delta_y;
	m_windowMapping.PixelDeltaToDelta(xdiff, ydiff);
	m_coords.x += xdiff;
	m_coords.y += ydiff;
	UpdateWindowMapper();
}

void LayerManager::UpdateWindowMapper()
{
	if(m_width > 0 && m_height > 0)
	{
		m_windowMapping.SetCoordinates(m_coords.y - m_zoomy/2, m_coords.x + m_zoomx/2, m_coords.x - m_zoomx/2, m_coords.y + m_zoomy/2);
		m_windowMapping.SetDestinationWindow(0, m_width, 0, m_height);
	}
}

//resets all logical coordinates, zoom, and cutoff.
void LayerManager::ResetCoords()
{
	m_coords.x = 0;
	m_coords.y = 0;
	m_zoomx = 20;
	m_zoomy = 20;
	UpdateWindowMapper();
}

void LayerManager::ChangeZoom(float zoom)
{
	zoom = MAX(0.1, zoom);
	m_zoomx = m_zoomy = zoom;
	assert(zoom > 0);
	UpdateWindowMapper();
}

void LayerManager::IncrementZoom(float deltazoom)
{
	ChangeZoom(m_zoomx + deltazoom);
}

void LayerManager::SetCoordinates(float x, float y, float zoomX, float zoomY)
{
	assert(zoomX > 0);
	assert(zoomY > 0);
	m_zoomy = zoomY;
	m_zoomx = zoomX;
	m_coords.x = x;
	m_coords.y = y;

	UpdateWindowMapper();
}

void LayerManager::IncrementParameter(float delta)
{
	int currentLayer = m_lbLayers->get_int_val();
	if(currentLayer >= 0 && currentLayer < m_renderableCount)
		m_renderableList[currentLayer]->IncrementParameter(delta);
}

void LayerManager::UpdateTextures()
{
	for(int i = 0; i < m_renderableCount; i++)
		m_renderableList[i]->UpdateTexture();
}

void LayerManager::Render()
{
	for(int i = 0; i < m_renderableCount; i++)
		m_renderableList[i]->Render( (((float)i)/MAX_RENDERABLE)/4 );

	m_infoDisplay.Render(0);
}


int LayerManager::AddLayer(Renderable * newLayer)
{
	if(m_renderableCount >= MAX_RENDERABLE)
	{
		//error, no more room
		return -1;
	}

	m_renderableList[m_renderableCount] = newLayer;
	newLayer->Initialize(this, m_width, m_height);
	char layername[200];
	sprintf_s(layername, "%s %d", newLayer->FriendlyName, ++m_TotalLayerCount);
	m_lbLayers->add_item(m_renderableCount, layername);

	m_renderableCount++;
}

void LayerManager::RemoveLayer()
{
	int index = m_lbLayers->get_int_val();

	if(index >= 0 && index < m_renderableCount)
	{
		delete m_renderableList[index];

		//move everything back
		for(int i = index; i < (m_renderableCount-1); i++)
			m_renderableList[i] = m_renderableList[i+1];

		m_renderableCount--;
		m_lbLayers->delete_item(index);
	}
}