#include <simplexnoise1234.h>
#include "NoiseSlice.h"

#include "LayerManager.h"


NoiseSlice::NoiseSlice() : Renderable()
{
	m_bitmapBits = NULL;
	m_cutoff = 0.5;
	
	m_Texture = 0;
}

void NoiseSlice::InternalInitialize()
{
	delete[] m_bitmapBits;
	m_bitmapBits = NULL;

	m_bitmapBits = new GLubyte[m_width*m_height];
}

NoiseSlice::~NoiseSlice(void)
{
	delete [] m_bitmapBits;
}

void NoiseSlice::UpdateTexture()
{
	// Create The Texture
	WriteNoiseBitmap();
	glGenTextures(1, &m_Texture);
	glBindTexture(GL_TEXTURE_2D, m_Texture);
	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);

	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);	// Linear Min Filter
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);	// Linear Mag Filter

	if(!m_bitmapBits)
	{
		GLubyte gray = 0x33;
		glTexImage2D(GL_TEXTURE_2D, 0, 1, 1, 1, 0, GL_LUMINANCE, GL_UNSIGNED_BYTE, &gray);
	}
	else
		glTexImage2D(GL_TEXTURE_2D, 0, 1, m_width, m_height, 0, GL_LUMINANCE, GL_UNSIGNED_BYTE, m_bitmapBits);

}

void NoiseSlice::ChangeParameter(float newParameter)
{
	m_cutoff = MAX(0.0f, newParameter);
	m_cutoff = MIN(1.0f, newParameter);
}

void NoiseSlice::IncrementParameter(float deltacutoff)
{
	ChangeParameter(m_cutoff + deltacutoff);
}

void NoiseSlice::WriteNoiseBitmap()
{
	GLubyte gray;

	for(int i = 0; i < m_width; i++)
	{
		for(int j = 0; j < m_height; j++)
		{
			float xVal = i, yVal = j;
			m_layerManager->m_windowMapping.PixelPositionToSpace(xVal, yVal);

			float noise = SimplexNoise1234::noise( xVal, yVal );

			float res = (noise + 1.0f)*0.5f;
			if(res > m_cutoff)
			{
				float scalefactor = (res - m_cutoff) * (1/(1 - m_cutoff));
				scalefactor = MIN(scalefactor, 1.0f);
				scalefactor = MAX(scalefactor, 0.0f);
				gray = (GLubyte)(255 * scalefactor);
			}
			else
				gray = 0;

			m_bitmapBits[i + m_width*j] = gray;
		}
	}
}

void NoiseSlice::Render(float zValue)
{
	glColor4f(1.0f,1.0f,1.0f, 1.0f);
	glBindTexture(GL_TEXTURE_2D, m_Texture);

	glBegin(GL_QUADS);
		// Front Face
		glTexCoord2f(0.0f, 0.0f); glVertex3f(0,		0,    zValue);
		glTexCoord2f(1.0f, 0.0f); glVertex3f(1.0f,	0.0f, zValue);
		glTexCoord2f(1.0f, 1.0f); glVertex3f(1.0f,  1.0f,  zValue);
		glTexCoord2f(0.0f, 1.0f); glVertex3f(0.0f,  1.0f,  zValue);
	glEnd();

}